@@ -5,12 +5,12 @@
 # Date: 2002-Nov-01 16:11 (EST)
 # Function: image drawing
 #
-# $Id: Image.pm,v 1.13 2003/01/28 18:40:07 jaw Exp $
+# $Id: Image.pm,v 1.17 2004/04/28 20:04:46 jaw Exp $
 
 package Image;
 use GD;
 use POSIX;
-use Encode;
+use Argus::Encode;
 
 sub new {
     my $c = shift;
@@ -68,11 +68,12 @@ sub axii {
     my $me = shift;
     my $im = $me->{img};
     # draw axii
-    
+
     $im->line( $me->xpt(-1), $me->ypt(-1), $me->xpt(-1), $me->ypt($me->{ymax}), $me->{color}{black});
     $im->line( $me->xpt(-1), $me->ypt(-1), $me->xpt($me->{xmax}), $me->ypt(-1), $me->{color}{black});
+    
     # 'Talking of axes,' said the Duchess, 'chop off her head!'
-    #   -- Aliced in Wonderland
+    #   -- Alice in Wonderland
 }
 
 sub border {
@@ -152,6 +153,8 @@ sub ydatapt {
     my $me = shift;
     my $pt = shift;
 
+    $pt = $pt < $me->{yd_min} ? $me->{yd_min} : $pt;
+    
     $me->ypt( ($pt - $me->{yd_min}) * $me->{yd_scale} );
 }
 
@@ -178,7 +181,10 @@ sub color {
 sub setrange {
     my $me = shift;
     my $r  = shift;
-    my( $l, $h ) = split /\s*-\s*/, $r;
+    my( $l, $h, $x ) = split /\s*-\s*/, $r;
+
+    ($l, $h) = (-$h, $x) if $l eq '';
+    $h = -$x if $h eq '';
 
     $me->{yd_min} = $l if defined($l) && $l ne '';
     $me->{yd_max} = $h if defined($h) && $h;
@@ -193,6 +199,8 @@ sub adjust {
     $me->{xmax} = $me->{xdim} - $me->{xmarr} - $me->{xmarl};
     $me->{ymax} = $me->{ydim} - $me->{ymarb} - $me->{ymart} ;
 
+    # print STDERR "adj $me->{ymin} > $me->{ymax} ; $me->{yd_min} > $me->{yd_max}\n";
+    
     $me->{xd_scale} = ($me->{xd_min} == $me->{xd_max}) ? 1
 	: $me->{xmax} / ($me->{xd_max} - $me->{xd_min});
 
@@ -208,6 +216,7 @@ sub analyze_samples {
     my( $st, $et, $min, $max );
     $st = $samp->[0]{time};
     $et = $samp->[-1]{time};
+    
     foreach my $s (@$samp){
 	$min = $s->{valu} if !defined($min) || $s->{valu} < $min;
 	$max = $s->{valu} if !defined($max) || $s->{valu} > $max;
@@ -584,7 +593,17 @@ sub draw {
     my $me   = shift;
     my $how  = shift;
     my $data = shift;
+
+    return unless @$data;
+
+    if( @$data < 3 ){
+        # $me->{img}->string(GD::gdMediumBoldFont, $me->{xmarl} + 4, 24, 'NOT ENOUGH DATA',
+	# 	      $me->{color}{red} );
+	return;
+    }
     
+    my $limit = 4 * ($me->{xd_max} - $me->{xd_min}) / @$data;
+	
     # 'What did they draw?' said Alice, quite forgetting her promise.
     #   -- Alice in Wonderland
     if( $how eq 'filled' ){
@@ -595,6 +614,8 @@ sub draw {
 	    my $x = $s->{time};
 	    my $y = $s->{valu} || $s->{ave};
 	    if( $me->xdatapt($x) - $me->xdatapt($px) > 1 ){
+		$px = $x - $limit if $x - $px > $limit;
+		
 		my $poly = GD::Polygon->new;
 		$poly->addPt($me->xdatapt($px), $me->ypt(0));
 		$poly->addPt($me->xdatapt($px), $me->ydatapt($py));
@@ -618,6 +639,8 @@ sub draw {
 	foreach my $s ( @$data ){
 	    my $x = $s->{time};
 	    my $y = $s->{valu} || $s->{ave};
+	    $px = $x - $limit if $x - $px > $limit;
+	    
 	    $me->{img}->line( $me->xdatapt($px), $me->ydatapt($py),
 			      $me->xdatapt($x),  $me->ydatapt($y),
 			      $me->color($s->{flag}) );
@@ -644,6 +667,8 @@ sub draw {
 	    }
 	    if( $me->xdatapt($x) - $me->xdatapt($px) > 1 ){
 		my $poly = GD::Polygon->new;
+		$px = $x - $limit if $x - $px > $limit;
+		
 		$poly->addPt($me->xdatapt($px), $me->ydatapt($pn));
 		$poly->addPt($me->xdatapt($px), $me->ydatapt($pm));
 		$poly->addPt($me->xdatapt($x),  $me->ydatapt($b));
