@@ -6,7 +6,7 @@
 # Date: 2002-Oct-31 15:45 (EST)
 # Function: server side of graph data collector
 #
-# $Id: graphd.pl,v 1.16 2008/09/14 15:37:56 jaw Exp $
+# $Id: graphd.pl,v 1.18 2011/11/03 02:13:41 jaw Exp $
 
 # read in data from argusd, save to data files
 # uses a large number of file descriptors, be sure
@@ -44,10 +44,10 @@ while( <> ){
 
     if( /^sample/ ){
 	# sample T obj st val samps hrs days
-	my(undef, $t, $name, $st, $val, $ss, $hs, $ds, #...
+	my(undef, $t, $name, $st, $val, $ss, $hs, $ds, $expect, $delta #...
 	   ) = split;
 	newobj( $name, $ss, $hs, $ds ) unless defined $obj{$name};
-	add_sample( $name, $t, $st, $val );
+	add_sample( $name, $t, $st, $val, $expect, $delta );
 	update_lru( $name );
     }
 }
@@ -59,6 +59,8 @@ sub add_sample {
     my $time = shift;
     my $stat = shift;
     my $valu = shift;
+    my $expt = shift;
+    my $delt = shift;
     my( $me, $fh, $off, $flg );
 
     $me = $obj{$name};
@@ -72,18 +74,19 @@ sub add_sample {
     # keep running average of delay
     $me->{pdelay} = $me->{delay};
     $me->{delay}  = ($me->{delay} + $delay) / 2 if $me->{lastt};
-    
+
     $off = $me->{sampl_index} * $SAMP_SIZE + $me->{sampl_start};
     sysseek( $fh, $off, 0 );
-    syswrite($fh, pack("NfNx4",
-		       $time, $valu, flags($stat)
+    syswrite($fh, pack("NfNff",
+		       $time, $valu, flags($stat), $expt, $delt
 		       ), $SAMP_SIZE);
-    
+
     $me->{sampl_index} ++;
     $me->{sampl_index} %= $me->{sampl_nmax};
     $me->{sampl_count} ++
 	unless $me->{sampl_count} >= $me->{sampl_nmax};
 
+
     # update hourly stats
     $me->{hours_min} = $valu
 	if !$me->{hours_nsamp} || ($valu < $me->{hours_min});
@@ -91,9 +94,12 @@ sub add_sample {
 	if !$me->{hours_nsamp} || ($valu > $me->{hours_max});
     $me->{hours_sigma} += $valu;
     $me->{hours_sigm2} += $valu * $valu;
+    $me->{hours_expect} += $expt;
+    $me->{hours_delta}  += $delt;
     $me->{hours_nsamp} ++;
     $me->{hours_flags} = sum_flags( $me->{hours_flags}, $stat );
 
+
     # update daily stats
     $me->{days_min} = $valu
 	if !$me->{days_nsamp}  || ($valu < $me->{days_min});
@@ -101,6 +107,8 @@ sub add_sample {
 	if !$me->{days_nsamp}  || ($valu > $me->{days_max});
     $me->{days_sigma} += $valu;
     $me->{days_sigm2} += $valu * $valu;
+    $me->{days_expect} += $expt;
+    $me->{days_delta}  += $delt;
     $me->{days_nsamp} ++;
     $me->{days_flags} = sum_flags( $me->{days_flags}, $stat );
 
@@ -115,12 +123,14 @@ sub add_sample {
 				   $me->{hours_nsamp});
 	$off = $me->{hours_index} * $HOURS_SIZE + $me->{hours_start};
 	sysseek( $fh, $off, 0 );
-	syswrite($fh, pack("NffffNNx4",
+	syswrite($fh, pack("NffffNNx4ff",
 			   $time,
 			   $me->{hours_min}, $me->{hours_max},
-			   $ave, $std, 
+			   $ave, $std,
 			   $me->{hours_nsamp},
 			   $me->{hours_flags},
+                           $me->{hours_expect} / $me->{hours_nsamp},
+                           $me->{hours_delta}  / $me->{hours_nsamp},
 			   ), $HOURS_SIZE);
 	$me->{hours_index} ++;
 	$me->{hours_index} %= $me->{hours_nmax};
@@ -128,9 +138,10 @@ sub add_sample {
 	    unless $me->{hours_count} >= $me->{hours_nmax};
 
 	# reset stats
-	$me->{hours_min} = $me->{hours_max} = 
-	    $me->{hours_nsamp} = $me->{hours_sigma} = 
-	    $me->{hours_sigm2} = $me->{hours_flags} = 0;
+	$me->{hours_min} = $me->{hours_max} =
+            $me->{hours_expect} = $me->{hours_delta} =
+	    $me->{hours_nsamp}  = $me->{hours_sigma} =
+	    $me->{hours_sigm2}  = $me->{hours_flags} = 0;
     }
 
     if( $me->{lastt} && ($lt[3] != $ct[3]) ){
@@ -140,12 +151,14 @@ sub add_sample {
 				   $me->{days_nsamp});
 	$off = $me->{days_index} * $DAYS_SIZE + $me->{days_start};
 	sysseek( $fh, $off, 0 );
-	syswrite($fh, pack("NffffNNx4",
+	syswrite($fh, pack("NffffNNx4ff",
 			   $time,
 			   $me->{days_min}, $me->{days_max},
-			   $ave, $std, 
+			   $ave, $std,
 			   $me->{days_nsamp},
 			   $me->{days_flags},
+                           $me->{days_expect} / $me->{days_nsamp},
+                           $me->{days_delta}  / $me->{days_nsamp},
 			   ), $DAYS_SIZE);
 
 	$me->{days_index} ++;
@@ -155,10 +168,11 @@ sub add_sample {
 
 	# reset stats
 	$me->{days_min} = $me->{days_max} =
-	    $me->{days_nsamp} = $me->{days_sigma} =
-	    $me->{days_sigm2} = $me->{days_flags} = 0;
+            $me->{days_expect} = $me->{days_delta} =
+	    $me->{days_nsamp}  = $me->{days_sigma} =
+	    $me->{days_sigm2}  = $me->{days_flags} = 0;
     }
-    
+
     $me->{lastt} = $time;
     $me->write_header();
 }
@@ -171,8 +185,8 @@ sub flags {
     my $st = shift;
 
     {
-	up => 0,
-	down => 1,
+	up       => 0,
+	down     => 1,
 	override => 2,
     }->{$st};
 }
@@ -192,13 +206,13 @@ sub update_lru {
 
     my $fp = int( $me->{pdelay} );
     my $fd = int( $me->{delay}  );
-    
+
     return if $fp == $fd;
-    
+
     # remove from pdelay list
     my @a = grep { $_ != $me } @{$lru[ $fp ]};
     $lru[ $fp ] = \@a;
-    
+
     # add to delay list
     push @{$lru[ $fd ]}, $me;
 }
@@ -209,7 +223,7 @@ sub close_lru {
 
     my @close;
     my $i = @lru;
-    
+
     while($i){
 	$i --;
 
@@ -239,7 +253,7 @@ sub openfile {
     my $err;
 
     my $file = "$DATADIR/gdata/$name";
-    
+
     if( -f $file ){
 	$x = "+< $file";
     }else{
@@ -255,7 +269,7 @@ sub openfile {
 		return;
 	    }
 	    $err = $!;
-	
+
 	    # close something
 	    close_lru();
 	}
@@ -272,7 +286,7 @@ sub openfile {
 	}
 	@lru = ();
     }
-    
+
     loggit( "$name on too tight, won't open: $err" );
     die "$err\n";
 }
@@ -282,7 +296,7 @@ sub newobj {
     my $ss   = shift;
     my $hs   = shift;
     my $ds   = shift;
-    
+
     my $me = $obj{$name} = {
 	name => $name,
     };
@@ -291,7 +305,7 @@ sub newobj {
 
     openfile($me);
     my $fh = $me->{fd};
-    
+
     binmode $fh;
     if( $me->{need_init} ){
 	$me->initfile($ss,$hs,$ds);
@@ -299,7 +313,7 @@ sub newobj {
     }else{
 	$me->initstats($ss,$hs,$ds);
     }
-    
+
 }
 
 ################################################################
@@ -318,9 +332,9 @@ sub initfile {
     $me->{sampl_nmax} = $ss if $ss;
     $me->{hours_nmax} = $hs if $hs;
     $me->{days_nmax}  = $ds if $ds;
-    
+
     $me->header_init();
-    
+
     # pre-extend
     truncate $fh, ($me->{days_start} + $DAYS_SIZE * $me->{days_nmax});
 
@@ -333,13 +347,20 @@ sub initstats {
     my $ss = shift;
     my $hs = shift;
     my $ds = shift;
-    my $fh = $me->{fd};
 
     my $magic = $me->read_header();
 
     unless( $magic eq $MAGIC){
-	loggit( "corrupt data file: $me->{name}" );
-	close $fh;
+
+        # do we need to upgrade?
+        $magic = $me->upgrade();
+
+        if( $magic eq $MAGIC ){
+            loggit("upgraded data file: $me->{name}");
+        }else{
+            loggit( "corrupt data file: $me->{name}" );
+            close $me->{fd};
+        }
     }
 
     # do we need to resize?
@@ -350,7 +371,7 @@ sub initstats {
 	loggit( "resizing data file: $me->{name}" );
 	$me->resize($ss,$hs,$ds);
     }
-    
+
 }
 
 sub ave_sdev {
