@@ -6,7 +6,7 @@
 # Date: 2002-Apr-02 12:09 (EST)
 # Function: argus startup and glue
 #
-# $Id: main.pl,v 1.70 2008/10/25 16:34:54 jaw Exp $
+# $Id: main.pl,v 1.77 2012/12/29 21:47:41 jaw Exp $
 
 # In God we trust, everyone else we monitor.
 #   -- U2 pilot motto
@@ -15,6 +15,7 @@ use lib('__LIBDIR__');
 BEGIN{
     require "conf.pl";
     require "misc.pl";
+    require "common.pl";
 }
 BEGIN{eval{require Diag;}}
 
@@ -38,20 +39,25 @@ use UserCron;
 use TestPort;
 use Argus::Ctl;
 use Argus::HashDir;
-use Resolv;
+use Argus::Schedule;
+use Argus::Ticket;
+use Argus::Dashboard;
+use Argus::Resolv;
 use DARP;
 
 BEGIN {
     # these may or may not be present
-    eval { require Tkt; };			# ticket system interface
+
 }
 
 use Sys::Syslog qw(:DEFAULT setlogsock);
 use POSIX qw(:errno_h setsid);
+use __DATABASE__;
 use Getopt::Std;
 
 getopts("a:c:dfg:htu:CDEHS:T");
 
+$DATABASE   = '__DATABASE__';
 $MAX_PERIOD = 300;
 $NAME = "Argus";
 
@@ -62,6 +68,7 @@ $idletime  = 0;
 $loopcount = 0;
 ($OSINFO = join(' ', (`uname -s`, `uname -r`, `uname -m`)))
     =~ s/[\n\r]//g;
+$COOKIE;
 
 my $childpid = 0;
 my $exitinprogress = 0;
@@ -100,7 +107,7 @@ Doc::describe_internal( $opt_H, $opt_S )if $opt_E;
 Doc::describe_control( $opt_H ) if $opt_C;
 
 if( $opt_t ){
-    Conf::readconfig( $opt_c || "$datadir/config" );
+    Conf::readconfigfiles( $opt_c || "$datadir/config" );
     exit $Conf::has_errors;
 }
 
@@ -163,7 +170,7 @@ MonEl::janitor() if -f "$datadir/stats/Top";
 
 # daemonize
 unless( $opt_f ){
-    # The victor daemon mounts obscure in air, 
+    # The victor daemon mounts obscure in air,
     # While the ship sails without the pilot's care.
     #   -- Virgil, Aeneid
     close STDIN;
@@ -178,6 +185,7 @@ unless( $opt_f ){
 
 $SIG{PIPE} = 'IGNORE';
 $SIG{HUP} = $SIG{QUIT} = $SIG{INT} = $SIG{TERM} = sub {
+    $^T = time();
     if( $childpid > 1 ){
 	kill "TERM", $childpid;
 	wait;
@@ -215,19 +223,20 @@ while(1){
 	exit;
     };
     $SIG{ALRM} = sub {};
-    
+
     # for children programs
     $ENV{ARGUS_PID}  = $$;
     $ENV{ARGUS_VER}  = $::VERSION;
     $ENV{ARGUS_DATA} = $datadir;
     $ENV{ARGUS_LIB}  = '__LIBDIR__';
-    
-    Conf::readconfig( $opt_c || "$datadir/config" );
+    $ENV{ARGUS_COOKIE} = $COOKIE = random_cookie(64);
+
+    Conf::readconfigfiles( $opt_c || "$datadir/config" );
     Doc->check_all();
 
     Control->Server::new_local( "$datadir/control", oct(topconf('chmod_control')) );
     $ENV{ARGUS_CTL} = "$datadir/control";
-    
+
     # load user specified perl code
     unshift @INC, "$datadir/perl";
     foreach my $m (split /\s+/, topconf('load_modules')){
@@ -236,7 +245,7 @@ while(1){
     }
     # run user specified program
     system( topconf('runatstartup') ) if topconf('runatstartup');
-    
+
     $^T = time();
     $starttime = $^T;
     loggit( "successful restart - $NAME running", 1 );
@@ -246,7 +255,7 @@ while(1){
         DB::reset();
 	rename "prof.out", "prof.out.startup";
     }
-    
+
     BaseIO::mainloop(
 		     maxperiod => $MAX_PERIOD,
 		     # run       => \&Prog::reap,	# annoys the profiler
