@@ -6,7 +6,7 @@
 # Date: 2002-Nov-01 11:54 (EST)
 # Function: lay paint on canvas. make pretty graphs.
 #
-# $Id: picasso.pl,v 1.20 2008/07/20 22:01:14 jaw Exp $
+# $Id: picasso.pl,v 1.29 2012/09/16 18:52:50 jaw Exp $
 
 # Computers are useless. They can only give you answers.
 #   -- Picasso
@@ -16,6 +16,7 @@ require "conf.pl";
 use Chart::Strip;
 use Argus::Encode;
 use Argus::Graph::Data;
+use POSIX;
 use strict;
 
 BEGIN {
@@ -81,6 +82,20 @@ my @colors  = qw(00FF00 DD33DD 00FFFF FFAAAA AAFFAA AAAAFF
 		 FFCC44 FF44CC 44CC88 88CC44 4488CC 8844CC);
 usecolors(decode($opt{gr_colors}));
 
+# barstyle: '', minmax, stddev
+# grstyle:  line, filled
+
+my $grstyle  = 'line';
+my $barstyle = $which eq 'samples' ? '' : $opt{barstyle};
+$barstyle = '' if $barstyle eq 'none';
+
+if( $size =~ /(\d+)x(\d+)/ ){
+    $opt{gr_width}  = $1;
+    $opt{gr_height} = $2;
+    $size = 'full';
+}
+
+my @dataopts;
 my @imgopts = (
 	       transparent      => $opt{transparent},
 	       draw_border      => $opt{drawborder},
@@ -92,46 +107,59 @@ my @imgopts = (
 	       limit_factor     => 4,
 	       );
 
+my $width;
 if( $size eq 'thumb' ){
-    push @imgopts, width => 160, height => 64;
+    push @imgopts, width => ($width = 200), height => 80;
     push @imgopts, draw_data_labels => 0, draw_tic_labels => 0;
+
+    push @dataopts, smooth   => $opt{gr_smooth};
+    push @dataopts, shadow   => {dx => 1, dy => 1, dw => 0} if $opt{gr_drop_shadow};
 }else{
     push @imgopts, height    => ($opt{gr_height} || 192);
-    push @imgopts, width     => 640, margin_right => 16;
+    push @imgopts, width     => ($width = $opt{gr_width} || 640);
+    push @imgopts, margin_right => 16;
     push @imgopts, title     => decode($opt{title});
     push @imgopts, x_label   => decode($opt{xlabel});
     push @imgopts, y_label   => decode($opt{ylabel});
-    push @imgopts, thickness => decode($opt{gr_line_thickness});
+
+    push @dataopts, thickness => decode($opt{gr_line_thickness});
+    push @dataopts, smooth    => $opt{gr_smooth};
+    push @dataopts, shadow    => {dx => 2, dy => 2, dw => 3} if $opt{gr_drop_shadow};
 }
 
 my $img = Chart::Strip->new( @imgopts );
 
-# barstyle: '', minmax, stddev
-# grstyle:  line, filled
-
-my $grstyle = ($which eq 'samples' && @ARGV == 1) ? 'filled' : 'line';
-my $barstyle = $which eq 'samples' ? '' : $opt{barstyle};
-$barstyle = '' if $barstyle eq 'none';
-
-
 my $colorn = 0;
 my $notenough = 0;
 foreach my $n (@ARGV){
     my $m = Argus::Graph::Data->new($n);
     my $color = color($colorn++);
     my $label = shift @labels;
-    
+
     if( $which eq 'samples' ){
-	$m->readsamples( $opt{gr_xrange_samples} );
-	
+        my $limit = $^T - ($opt{gr_xrange_samples} || 36*3600);
+	$m->readsamples( $limit );
+
 	if( @{$m->{samples}} < 2 ){
 	    $notenough ++;
 	    next;
 	}
-	$img->add_data( $m->{samples}, {style => $grstyle, color => $color, label => $label} );
+
+        subsample( $m, $width );
+
+        my $hwab;
+        $hwab = add_hw_prediction($m) if @ARGV==1 && $size ne 'thumb' && $opt{gr_show_hwab};
+        push @dataopts, shadow => undef if $hwab;
+
+	$img->add_data( $m->{samples}, {style => $grstyle, color => $color, label => $label,
+                                        @dataopts,
+                        } );
+
     }else{
+        my $limit = $^T - ($opt{"gr_xrange_$which"} ||($which eq 'hours' ? 180 * 3600 : 90 * 24 * 3600));
+
 	# limit to either 576 points, or specified range
-	$m->readsummary( $which, 576, $opt{"gr_xrange_$which"} );
+	$m->readsummary( $which, 576, $limit );
 	# print STDERR "data: ", scalar(@{$m->{samples}}), "\n";
 
 	if( @{$m->{samples}} < 2 ){
@@ -139,18 +167,17 @@ foreach my $n (@ARGV){
 	    next;
 	}
 
-	if( $barstyle eq 'minmax' ){
-	    $img->add_data( $m->{samples}, {style => 'range', color => 'blue'} );
-	}
-	if( $barstyle eq 'stddev' ){
-	    foreach my $s ( @{$m->{samples}} ){
-		$s->{min} = $s->{ave} - $s->{stdv};
-		$s->{max} = $s->{ave} + $s->{stdv};
-	    }
-	    $img->add_data( $m->{samples}, {style => 'range', color => 'blue'} );
-	}
-	
-	$img->add_data( $m->{samples}, {style => $grstyle, color => $color, label => $label} );
+        subsample( $m, $width );
+
+        add_bars($m) if $barstyle && $barstyle ne 'none' && $size ne 'thumb';
+
+        my $hwab;
+        $hwab = add_hw_prediction($m) if @ARGV==1 && $size ne 'thumb' && $opt{gr_show_hwab};
+        push @dataopts, shadow => undef if $hwab;
+
+	$img->add_data( $m->{samples}, {style => $grstyle, color => $color, label => $label,
+                                        @dataopts,
+                        } );
     }
 }
 
@@ -188,18 +215,116 @@ if( $opt{filetype} eq 'gif' ){
 }
 exit 0;
 
+################################################################
+
+sub add_bars {
+    my $m = shift;
+
+    my @data;
+
+    if( $barstyle eq 'minmax' ){
+        @data = map {
+            {
+                time => $_->{time},
+                min  => $_->{min},
+                max  => $_->{max},
+            }
+        } @{$m->{samples}};
+    }
+    if( $barstyle eq 'stddev' ){
+        my $n = $opt{gr_bar_nstddev} || 1;
+
+        @data = map {
+            {
+                time => $_->{time},
+                min  => $_->{ave} - $n * $_->{stdv},
+                max  => $_->{ave} + $n * $_->{stdv},
+            }
+        } @{$m->{samples}};
+    }
+
+    $img->add_data( \@data, {style => 'range', color => $opt{gr_bar_color}} );
+}
+
+
+# include holt-winters prediction on graph?
+sub add_hw_prediction {
+    my $m = shift;
+
+    my $enabled;
+    my @data = map {
+        $enabled = 1 if $_->{expect};
+        {
+            time => $_->{time},
+            min  => $_->{expect} - $_->{delta},
+            max  => $_->{expect} + $_->{delta},
+      };
+    } @{$m->{samples}};
+
+    return unless $enabled;
+
+    # remove initial 0s (probably wasn't enabled yet)
+    my $z;
+    @data = grep {
+        $z || $_->{max} && ($z=1)
+    } @data;
+
+    $img->add_data( \@data, {
+        style     => 'range',
+        color     => $opt{gr_hwab_color},
+        label     => '',
+        shadow    => undef,
+
+    } );
+
+    # prevent this data from being considered in determining the graph ranges
+    # chart::strip should add a feature
+    $img->{xd_max} = $img->{xd_min} = $img->{yd_max} = $img->{yd_min} = undef
+      if $opt{gr_hwab_range_ignore};
+
+    1;
+}
+
+sub subsample {
+    my $m     = shift;
+    my $width = shift;
+
+    my $samp = $m->{samples};
+    if( @$samp > $width/2 ){
+        # more samples than we have room for. subsample
+        my $n = ceil( 2 * @$samp / $width );
+
+        my @s;
+        for my $i (0 .. $#{$samp}){
+            next if $i % $n;
+
+            my $ct = 1;
+            my $tt = $samp->[$i]{value};
+            for my $j (0 .. $n/2){
+                if( $i-$j >= 0 ){ $ct ++; $tt += $samp->[$i-$j]{value}; }
+                if( $i+$j <= $#{$samp} ){ $ct ++; $tt += $samp->[$i+$j]{value}; }
+            }
+
+            $samp->[$i]{value} = $tt / $ct;
+            push @s, $samp->[$i];
+        }
+
+        $m->{samples} = \@s;
+    }
+}
+
 sub error {
     my $msg = shift;
     my $im;
 
     print STDERR "picasso: ERROR $msg\n";
-    
-    $im = new GD::Image(160, 64);
+
+    $im = new GD::Image(200, 80);
     my $blk = $im->colorAllocate(0,0,0);
     my $red = $im->colorAllocate(0xFF,0x88,0x88);
 
-    $im->filledRectangle(0,0, 159,63, $red);
-    $im->rectangle(0,0, 159,63, $blk);
+    $im->filledRectangle(0,0, 199,79, $red);
+    $im->rectangle(0,0, 199,79, $blk);
     my $grtitle = $opt{errortitle} || 'PICASSO: GRAPH ERROR';
     $im->string(GD::gdMediumBoldFont, 4, 2, $grtitle, $blk);
     my @a = grep {$_ || ()} split /(.{25})/, $msg;
@@ -209,7 +334,7 @@ sub error {
 	$im->string(GD::gdSmallFont, 4, $y, $_, $blk);
 	$y += 10;
     }
-    
+
     if( $opt{filetype} eq 'gif' ){
 	print $im->gif();
     }else{
@@ -226,9 +351,9 @@ sub color {
     $colors[ $n % @colors ];
 }
 
-# Nor long the sun his daily course withheld, 
-# But added colors to the world reveal'd: 
-# When early Turnus, wak'ning with the light, 
+# Nor long the sun his daily course withheld,
+# But added colors to the world reveal'd:
+# When early Turnus, wak'ning with the light,
 #   -- Virgil, Aeneid
 sub usecolors {
     my $cs = shift;
@@ -271,7 +396,7 @@ sub logoize {
     [3, 21, 0], [1, 21, 0], [1, 19, 3], [2, 18, 0], [1, 17, 0], [2, 16, 2], [1, 14, 3], [4, 13, 0],
     [2, 13, 0], [0, 13, 0], [4, 12, 0], [2, 12, 0], [0, 12, 0], [3, 11, 0],  [3, 9, 0],  [4, 8, 0],
     [3, 7, 0],   [1, 6, 3],  [1, 4, 3],  [4, 3, 0],  [2, 3, 0],  [4, 2, 0],  [1, 2, 0],  [2, 1, 1] );
-    
+
     foreach my $l (@data){
 	foreach my $i ($l->[0] .. $l->[0] + $l->[2]){
 	    $im->setPixel($img->{width} - 9 + $i, 2 + $l->[1], $img->{color}{blue});
